using System;

class ElectronicDevice
{
    private string brandName;
    private bool isOn;
    
    public ElectronicDevice()
    {
        brandName = "unknown";
        isOn = false;
    }
    
    public ElectronicDevice(string initBrandName)
    {
        brandName = initBrandName;
        isOn = false;
    }
    
    public string BrandName
    {
        get
        {
            return brandName;
        }
    
        set
        {
            brandName = value;
        }
    }
    
    public virtual void SwitchOn()
    {
        isOn = true;
        Console.WriteLine("On");
    }
    
    public virtual void SwitchOff()
    {
        isOn = false;
        Console.WriteLine("Off");
    }
}

class Radio : ElectronicDevice
{
    private double currentFrequency;
    
    public Radio() : base()
    {
        currentFrequency = 0;
    }
    
    public Radio(double initCurrentFrequency, string initBrandName) : base(initBrandName)
    {
        currentFrequency = initCurrentFrequency;
    }

    public override void SwitchOn()
    {
        base.SwitchOn();
        Console.WriteLine("Radio");
    }
    
    public override void SwitchOff()
    {
        base.SwitchOff();
        Console.WriteLine("Radio");
    }
}

class Computer : ElectronicDevice
{
    private int internalMemory;

    public Computer() : base()
    {
        internalMemory = 0;
    }
    
    public Computer(int initInternalMemory, string initBrandName) : base(initBrandName)
    {
        internalMemory = initInternalMemory;
    }
    
    public override void SwitchOn()
    {
        base.SwitchOn();
        Console.WriteLine("Computer");
    }
    
    public override void SwitchOff()
    {
        base.SwitchOff();
        Console.WriteLine("Computer");
    }
}

class MobilePhone : ElectronicDevice
{
    private uint lastNumberDialled;

    public MobilePhone() : base()
    {
        lastNumberDialled = 0;
    }

    public MobilePhone(uint initLastNumberDialled, string initBrandName) : base(initBrandName)
    {
        lastNumberDialled = initLastNumberDialled;
    }

    public override void SwitchOn()
    {
        base.SwitchOn();
        Console.WriteLine("Mobile Phone");
    }
    
    public override void SwitchOff()
    {
        base.SwitchOff();
        Console.WriteLine("Mobile Phone");
    }
}

class LaptopComputer : Computer
{
    private uint maxBatteriLife;
    
    public LaptopComputer() : base()
    {
        maxBatteriLife = 0;
    }
    
    public LaptopComputer(uint initMaxBatLife, int initInternalMemory, string initBrandName) : base(initInternalMemory, initBrandName)
    {
        maxBatteriLife = initMaxBatLife;
    }
}

class Tester
{
    public static void Main()
    {
        Radio myRadio = new Radio(100, "Bang & Olufsen");
        Console.WriteLine("BrandName: " + myRadio.BrandName);
        
        LaptopComputer myLaptop = new LaptopComputer(12, 256, "IBM");
        Console.WriteLine("BrandName: " + myLaptop.BrandName);
    }
}
