using System;

class GameController
{
    public delegate void MoveRequest(object sender, MoveRequestEventArgs e);
    
    public event MoveRequest OnMoveRequest;
    
    Car[] gameCars = new Car[10];
    Motorbike[] gameMotorbikes = new Motorbike[10];
    
    string name;
    int speedParam = 0;
    int motorbikeCounter = 0;
    int carCounter = 0;
    int carNumber = 0;
    int motorbikeNumber = 0;
    
    public void Run()
    {
        string answer;
        Console.WriteLine("Please select from the following menu: ");
        Console.WriteLine("A)dd new car");
        Console.WriteLine("B) Add new motorbike");
        Console.WriteLine("C) Subscribe car to events");
        Console.WriteLine("M) Subscribe motorbike to events");
        Console.WriteLine("U)nsubscribe car from events");
        Console.WriteLine("V) Unsubscribe motorbike from events");
        Console.WriteLine("L)ist cars and motorbikes in current game");
        Console.WriteLine("F)ast forward");
        Console.WriteLine("S)low forward");
        Console.WriteLine("R)everse");
        Console.WriteLine("T)erminate");

        do
        {
            Console.WriteLine("Select new option:");
            answer = Console.ReadLine().ToUpper();
            
            switch(answer)
            {
                case "A":
                    Console.Write("Enter name of the new car: ");
                    name = Console.ReadLine();
                    Console.Write("Enter car speed parameter of the new car: ");
                    speedParam = Convert.ToInt32(Console.ReadLine());
                    gameCars[carCounter] = new Car(speedParam, name);
                    carCounter++;
                break;
                case "B":
                    Console.Write("Enter name of the new motorbike: ");
                    name = Console.ReadLine();
                    gameMotorbikes[motorbikeCounter] = new Motorbike(name);
                    motorbikeCounter++;
                break;
                case "C":
                    Console.Write("Enter array index of car you want to subscribe to events: ");
                    carNumber = Convert.ToInt32(Console.ReadLine());
                    gameCars[carNumber].Subscribe(this);
                break;
                case "M":
                    Console.Write("Enter array index of motorbike you want to subscribe to events: ");
                    motorbikeNumber = Convert.ToInt32(Console.ReadLine());
                    gameMotorbikes[motorbikeNumber].Subscribe(this);
                break;
                case "U":
                    Console.Write("Enter array index of car you want to unsubscribe from events: ");
                    carNumber = Convert.ToInt32(Console.ReadLine());
                    gameCars[carNumber].Unsubscribe(this);
                break;
                case "V":
                    Console.Write("Enter array index of motorbike you want to unsubscribe from events: ");
                    motorbikeNumber = Convert.ToInt32(Console.ReadLine());
                    gameMotorbikes[motorbikeNumber].Unsubscribe(this);
                break;
                case "L":
                    Console.WriteLine("Cars currently listed:");
                    for(int i=0; i < carCounter; i++)
                    {
                        Console.WriteLine(gameCars[i]);
                    }
                    Console.WriteLine("Motorbikes currently listed:");
                    for(int i=0; i < motorbikeCounter; i++)
                    {
                        Console.WriteLine(gameMotorbikes[i]);
                    } 
                break;
                case "F":
                    if (OnMoveRequest != null)
                        OnMoveRequest(this, new MoveRequestEventArgs(MoveRequestType.FastForward));
                break;
                case "S":
                    if (OnMoveRequest != null)
                        OnMoveRequest(this, new MoveRequestEventArgs(MoveRequestType.SlowForward));
                break;
                case "R":
                    if (OnMoveRequest != null)
                        OnMoveRequest(this, new MoveRequestEventArgs(MoveRequestType.Reverse));
                break;
                case "T":
                break;
                default:
                    Console.WriteLine("Invalid choice. Please try again");
                break;
            }
        } while(answer != "T");
    }
}

class Motorbike
{
    private int distance;
    private string name;
    const int speedParam = 30;
    
    public Motorbike(string initName)
    {
        distance = 0;
        name = initName;
    }
    
    public void Subscribe(GameController controller)
    {
        controller.OnMoveRequest += new GameController.MoveRequest(MoveRequestHandler);
    }
    
    public void Unsubscribe(GameController controller)
    {
        controller.OnMoveRequest -= new GameController.MoveRequest(MoveRequestHandler);
    }
    
    public void MoveRequestHandler(object sender, MoveRequestEventArgs e)
    {
        switch (e.Request)
        {
            case MoveRequestType.SlowForward:
                distance += speedParam;
                Console.WriteLine("Motorbike name: " + name + " Moving forward. Distance: " + distance);
            break;
            case MoveRequestType.FastForward:
                distance += speedParam; 
                Console.WriteLine("Motorbike name: " + name + " Moving forward. Distance: " + distance);
            break;
            case MoveRequestType.Reverse:
                distance -= 3;
                Console.WriteLine("Motorbike name: " + name + " Reversing. Distance: " + distance);
            break;
        }
    }
    
    public override string ToString()
    {
        return name;   
    } 
}

